% Example demonstrating the use of the RoundCut for segmenting multiple
% surface in three settings:
%   1. only surface costs are used (one for each of the 2 surfaces)
%   2. only region costs are used (one for each of the 3 regions)
%   3. both surface and the region costs are used
% Visualization of the intermediate results is commented out, but has not
% been removed from the script.

clear, close all
addpath 'round_cut' 'GraphCut' 'auxiliary_functions'...
    'mesh_functions' 'visualization_functions' 'cost_functions' 'test_cases'

V = citrus_press_regions;

nu = 10; % subdivistion frequency for polyhedra
sphere = sphere_tiling(nu);
sphere = translate_mesh(sphere,(size(V)+1)/2); % placing sphere in volume center
q = 1:0.5*norm(size(V)); % sampling the whole radius

U = sample_volume(V,sphere,q,0); % sampled volume
%figure, imagesc(U), colormap gray

delta_r = [100 100]; % flexibility of found surfaces
delta_lu = [1 100]; % distance between the found surfaces

%% SURFACE DETECTION USING SURFACE COST ONLY

cost_s(:,:,1) = cost_S_biggest_increase(U); % first surface
cost_s(:,:,2) = cost_S_biggest_decrease(U); % second surface
cost_r = zeros([size(U),3]); % no region cost

%figure
%for k = 1:size(cost_s,3)
%    subplot(1,size(cost_s,3),k), imagesc(cost_s(:,:,k)), colormap gray    
%end
sk_S = optimal_k_cut(cost_s,cost_r,sphere.adjacency,delta_r,delta_lu) + 0.5;

%figure, imagesc(U), colormap gray, hold on, plot(1:size(sk_S,1)'*[1 1],sk_S)

SK = fold_surface(sk_S,sphere,q);
resultS = change_geometry(sphere,SK,false);

figure, h = draw_mesh(resultS);
set(h{1},'FaceColor','r','FaceAlpha',0.7)
set(h{2},'FaceColor','b','FaceAlpha',0.3)
set([h{1},h{2}],'EdgeColor','none','FaceLighting','phong','BackFaceLighting','unlit')
axis vis3d equal off, view(3), camlight right

% figure, animate_volume_and_contours(V,resultS,0,'x','nonempty')

%% SURFACE DETECTION USING REGION COST ONLY

cost_s_empty = zeros([size(U),2]); % no surface cost
mu = [0.6 0.9 0.1];
sigma = [0.01 0.02 0.001].^0.5;
cost_r = cost_R_normalized_probability(U,mu,sigma); % region cost

%figure
%for k = 1:size(cost_r,3)
%    subplot(1,size(cost_r,3),k), imagesc(cost_r(:,:,k))
%end

sk_R = optimal_k_cut(cost_s_empty,cost_r,sphere.adjacency,delta_r,delta_lu) + 0.5;

%figure, imagesc(U), colormap gray, hold on, plot(1:size(sk_R,1)'*[1 1],sk_R)

SK = fold_surface(sk_R,sphere,q);
resultR = change_geometry(sphere,SK,false);

figure, h = draw_mesh(resultR);
set(h{1},'FaceColor','r','FaceAlpha',0.7)
set(h{2},'FaceColor','b','FaceAlpha',0.3)
set([h{1},h{2}],'EdgeColor','none','FaceLighting','phong','BackFaceLighting','unlit')
axis vis3d equal off, view(3), camlight right

%figure, animate_volume_and_contours(V,resultS,0,'x','nonempty')

%% SURFACE DETECTION USING BOTH THE SURFACE AND THE REGION COST
sk_B = optimal_k_cut(cost_s,cost_r,sphere.adjacency,delta_r,delta_lu) + 0.5;

%figure, imagesc(U), colormap gray, hold on, plot(1:size(sk_B,1)'*[1 1],sk_B)

SK = fold_surface(sk_B,sphere,q);
resultB = change_geometry(sphere,SK,false);

figure, h = draw_mesh(resultB);
set(h{1},'FaceColor','r','FaceAlpha',0.7)
set(h{2},'FaceColor','b','FaceAlpha',0.3)
set([h{1},h{2}],'EdgeColor','none','FaceLighting','phong','BackFaceLighting','unlit')
axis vis3d equal off, view(3), camlight right
 
% figure, animate_volume_and_contours(V,resultB,0,'x','nonempty')

