function mesh_new = mesh_subdivision(mesh,iter,mode)
% mesh.verticess - list of vertices
% mesh.faces - list of faces
% iter - number of subdivision iterations
% mode - 'loop' or 'butterfly' or 'none'
% mesh_subdivision does not handle boundary vertices
%
% Author: Vedrana Andersen Dahl, vand@dtu.dk 
% Date: 2.7.2013.

tic, fprintf('Mesh subdivision ... ')

verts = mesh.vertices;
faces = mesh.faces;

for k = 1:iter
    V = size(verts,1);
    F = size(faces,1);        
    
    C = faces_to_circulator(faces); % edges and opposite vertices
    [i,j] = find(triu(C)); % undirected edges
    E = length(i);
        
    % output
    verts_new = [verts;zeros(E,3)]; % new vertices list
    faces_new = zeros(4*F,3); % new faces list
    
    % matrix which links each edge with the index of the added vertex
    H = sparse(i,j,(1:E)+V,V,V);    
    
    H = H + H'; % copying UT elements to LT
    
    % new vertices (on the old edges)
    for e = 1:E
        % two old vertices opposite of the edge e        
        opp1 = C(i(e),j(e));
        opp2 = C(j(e),i(e));
        % using subdivision rules to calculate position of added vertices
        if strcmp(mode,'butterfly')
            % finding four vertices on outer faces
            v1 = C(i(e),opp1);
            v2 = C(opp2,i(e));
            v3 = C(opp1,j(e));
            v4 = C(j(e),opp2);            
            % updating position of the added vertex (index e+V)
            verts_new(e+V,:) = (8/16)*sum(verts([i(e) j(e)],:)) + ...
                (2/16)*sum(verts([opp1 opp2],:)) - (1/16)*sum(verts([v1 v2 v3 v4],:));
        elseif strcmp(mode,'loop')
            verts_new(e+V,:) = (3/8)*sum(verts([i(e) j(e)],:)) + ...
                (1/8)*sum(verts([opp1 opp2],:));
        else
            verts_new(e+V,:) = mean(verts([i(e) j(e)],:));
        end
    end
   
    if strcmp(mode,'loop')
        % updating positions of old vertices
        for v = 1:V
            verts_new(v,:) = (5/8)*verts_new(v,:) + ...
                (3/8)*mean(verts_new(H(v,find(H(v,:))),:));          
        end
    end
    
    for f = 1:F
        faces_new((f-1)*4+1,:) = [faces(f,1) H(faces(f,1),faces(f,2))...
            H(faces(f,1),faces(f,3))];
        faces_new((f-1)*4+2,:) = [H(faces(f,2),faces(f,1)) faces(f,2) ...
            H(faces(f,2),faces(f,3))];
        faces_new((f-1)*4+3,:) = [H(faces(f,3),faces(f,1)) ...
            H(faces(f,3),faces(f,2)) faces(f,3)];
        faces_new((f-1)*4+4,:) = [H(faces(f,2),faces(f,3)) ...
            H(faces(f,3),faces(f,1)) H(faces(f,1),faces(f,2))];
    end    
    
    % for looping
    verts = verts_new;
    faces = faces_new;
    
end

mesh_new.vertices = verts;
mesh_new.faces = faces;
mesh_new.adjacency = faces_to_circulator(faces)>0;
mesh_new.normals = vertex_normals(verts,faces); 

disp(['finished at ',num2str(toc)])

end