function mesh = sphere_tiling(nu)
% nu - subdivistion frequency
% Resulting number of vertices will be V = v + f*(nu+1)*(nu-1)/2,
% starting from isosahedron where v = 12 and f = 20.
% mesh.vertices - list of vertices on a unit sphere
% mesh.faces
% mesh. adjacency - adjacency matrix defining triangle mesh on S
% mesh.normals
%
% Author: Vedrana Andersen, vand at dtu.dk
% Date: 29.5.2012.

tic, fprintf('Sphere tiling ... ')
[S,A] = make_isosahedron;
[S,A] = frequency_subdivision(S,A,nu);
% [S,A] = sort_vertices(S,A); was used for visualization 

mesh.vertices = S;
mesh.faces = adjacency_to_faces(S,A);
mesh.adjacency = sparse(A);
mesh.normals = S;

disp(['finished at ',num2str(toc)])

end

function [S,A] = make_isosahedron
% Makes an isosahedron on a unit sphere

phi=(1+sqrt(5))/2;

S = [0 -1 -phi
    0 -1 phi
    0 1 phi
    0 1 -phi
    -1 -phi 0
    -1 phi 0
    1 phi 0
    1 -phi 0
    -phi 0 -1
    phi 0 -1
    phi 0 1
    -phi 0 1];

S = S/(1+phi^2)^0.5;

A = false(12,12);

A(1,[4 5 8 9 10]) = true;
A(2,[3 5 8 11 12]) = true;
A(3,[2 6 7 11 12]) = true;
A(4,[1 6 7 9 10]) = true;
A(5,[1 2 8 9 12]) = true;
A(6,[3 4 7 9 12]) = true;
A(7,[3 4 6 10 11]) = true;
A(8,[1 2 5 10 11]) = true;
A(9,[1 4 5 6 12]) = true;
A(10,[1 4 7 8 11]) = true;
A(11,[2 3 7 8 10]) = true;
A(12,[2 3 5 6 9]) = true;

end

function [S_new, A_new] = frequency_subdivision(S, A, nu)
% S - list of vertices on a unit sphere
% A - adjacency matrix defining triangle mesh on S
% nu - subdivistion frequency

if nu>1
    
    V_old = size(S,1);
    [i,j] = find(triu(A)); % edges (i,j), i<j
    E = numel(i);
    
    % making a list of faces
    faces = zeros(2*E,3);
    for e = 1:E
        opp = find(A(i(e),:) & A(j(e),:)); % indices of two vertices opposite e
        faces(2*e-1:2*e,:) = sort([[i(e) j(e); i(e) j(e)] opp'],2);
    end
    faces = unique(faces,'rows'); % there is E*2/3 faces
    
    % PREALOCATING
    % every edge contributes with (nu-1) new vertices
    % every face contributes with (nu-1)*(nu-2)/2 new vertices
    
    V_add =  E*(nu-1) + (E*2/3)*(nu-2)*(nu-1)/2;
    S_new = [S; zeros(V_add,3)]; %  new vertex list
    A_new = false(V_old + V_add); % new adjacency matrix (square and empty)
    H = zeros(V_old,V_old,nu-1); % to link edges with indices of added vertices
    
    % ADDING VERTICES ON EXISTING EDGES
    last_used_vertex = V_old;
    for e = 1:E
        % puts indices of added vertices in H
        z(1,1,:) = (1:nu-1) + last_used_vertex;
        H(i(e),j(e),:) = z; % using only upper triangular part
        % updating positions of the added vertices
        S_new((1:nu-1) + last_used_vertex,:) = ...
            calculate_positions(S([i(e);j(e)],:),nu-1);
        % linking the new edges
        A_new(i(e), last_used_vertex+1) = true;
        for s = last_used_vertex+1 : last_used_vertex+nu-2
            A_new(s, s+1) = true;
        end
        A_new(last_used_vertex+nu-1, j(e)) = true;
        last_used_vertex = last_used_vertex+(nu-1);
    end
    
    % ADDING VERTICES INSIDE THE EXISTING FACES
    % There is (nu+2)*(nu+1)/2 vertices (on corners, edges and inside)
    % involved in each face, of those (nu-1)*(nu-2)/2 inside vertices.
    % Indices of all those vertices are first listed in a layered order,
    % to assist linking the new edges.
    for f = 1:size(faces,1)
        index = zeros((nu+2)*(nu+1)/2,1); % placeholder for indices list
        % placing indices of existing vertices (on corners and edges)
        index(1) = faces(f,1); % first corner
        index(2:nu) = H(faces(f,1),faces(f,2),:); % bottom edge
        index(nu+1) = faces(f,2); % second corner
        c = nu+2; % first index in first layer
        for k = 1:nu-1 % inside layers to be added
            index(c)= H(faces(f,1),faces(f,3),k); % first vertex in current layer
            index(c+nu-k) = H(faces(f,2),faces(f,3),k); % last vertex in current layer
            c = c+nu-k+1; % first index in next layer
        end
        index(c) = faces(f,3); % third corner
        
        % assigning indices to new vertices (inside)
        index(index == 0) = (1:(nu-1)*(nu-2)/2) + last_used_vertex;
        last_used_vertex = last_used_vertex + (nu-1)*(nu-2)/2;
        
        % updating position for new vertices and linking
        c = nu + 2; % first index in first layer
        for k = 1:nu-1
            for s = c : c+nu-k-1
                A_new(index(s), index(s+1)) = true; % linking right
                A_new(index(s+k-nu-1), index(s)) = true; % linking down left
                A_new(index(s+k-nu-1), index(s+1)) = true; % linking down right
            end
            s = (c+1:c+nu-k-1)'; % indices of inside vertices
            % updating positions of the added (inside) vertices
            S_new(index(s),:) = ...
                calculate_positions(S_new(index([c;c+nu-k]),:),nu-k-1);
            c = c+nu-k+1; % first index in next layer
        end
    end
    % edges linking in both directions
    A_new = A_new | A_new';
   
else % if nu == 1
    
    S_new = S;
    A_new = A;
    
end

end

function inbetween = calculate_positions(endpoints,n)
% inbetween are positions of n vertices inserted in between endpoints
% along the minor arc of the great circle

x = cross(endpoints(1,:),endpoints(2,:)); % rotation axis
x = x/norm(x);
y = cross(x,endpoints(1,:)); % third coordinate vector
y = y/norm(y);

% rotation angles
theta = (1:n)'*acos(dot(endpoints(1,:),endpoints(2,:)))/(n+1);

% rotating endpoint(1,:) in the direction of endpoint(2,:) for angles theta
inbetween = cos(theta)*endpoints(1,:) + sin(theta)*y;

end

function faces = adjacency_to_faces(verts,A)

[i,j] = find(triu(A)); % edges (i,j), i<j
E = numel(i);

faces = zeros(2*E,3);
for e = 1:E
    opp = find(A(i(e),:) & A(j(e),:)); % indices of two vertices opposite e
    faces(2*e-1:2*e,:) = sort([[i(e) j(e); i(e) j(e)] opp'],2);
end
faces = unique(faces,'rows');
faces = orient_faces(verts,faces);

end

function faces = orient_faces(verts,faces)
% orienting the faces so that normal points outwards, in a naive way by
% dotting with the radial vector
q = faces';
verts = verts - repmat(mean(verts,1),[size(verts,1),1]);
% each row of p has coordinates [xa ya za xb yb zb xc yc zc] for each face
p = reshape((verts(q(:),:))',[9,size(faces,1)])';
% looking at the sign of dot product between face normal and face centrum
d = p(:,1).*p(:,5).*p(:,9) - p(:,1).*p(:,8).*p(:,6) + ...
    p(:,4).*p(:,8).*p(:,3) - p(:,4).*p(:,2).*p(:,9) + ...
    p(:,7).*p(:,2).*p(:,6) - p(:,7).*p(:,5).*p(:,3);
faces(d<0,[2 3]) = faces(d<0,[3 2]); % exchanging indices

end




