function s = optimal_cut(cost,A,delta_r)
% Finds an optimal cut (a surface) between lower and upper part of a volume.
%
% COST is a size(R,N) collection of samples in columns. Values in cost are
%       an inverse to likelihood of a sample (r,n) containing surface.
% A is a size(N,N) adjacency matrix for cost.
% DELTA_R is a surface stiffness parameter.
% S is a length N array of r coordinates for the segmented surface.
%
% Author: Vedrana Andersen, vand at dtu.dk
% Date: 30.5.2013.
%
% NOTE: needs Kolmogorovns implementation of the algorithm from
%		"An Experimental Comparison of Min-Cut/Max-Flow Algorithms for
%       Energy Minimization in Vision."
%		Yuri Boykov and Vladimir Kolmogorov. PAMI 2004

tic, fprintf('Optimal cut, ')

% assigning an unique index to each pixel in cost
[R,N] = size(cost);
fprintf('N=%d, M=%d ... ',N,R)

indices = reshape(1:R*N,[R N]);
base = indices(1,:);

% EDGES WHICH ARE THE SAME FOR ALL SURFACES:
% intracolumn arcs pointing down
ea = connect_verticaly(indices,-1);
Ea = [ea,ones(size(ea,1),1)*[inf,0]]; % assigning [inf 0] weight

% base edges, part of intercolumn arcs
eb = unique(sort(connect_to_neighbors(base,A,0),2),'rows');

% assigning [inf inf] weight, base edges are in both directions
Eb = [eb,ones(size(eb,1),1)*[inf,inf]];

% EDGES WHICH DEPEND ON SURFACE STIFFNESS:
% slanted edges, part of intercolumn arcs
ed = connect_to_neighbors(indices,A,-delta_r);
% assigning [inf 0] weight, slanted edges are in one direction
Ed = [ed,ones(size(ed,1),1)*[inf,0]];

E = [Ea; Eb; Ed]; % all intracolumn and intercolumn edges

% EDGES WHICH DEPEND ON SURFACE LIKELIHOOD:
% up to here we do not use cost -- consider efficient algorithm if solving
% multiple problems of the same size
% vertex cost done simulatniously for all surfaces
w = -1*ones(size(cost)); % to prevent empty solution, see second half of section 4.1
w(2:end,:) = double(cost(2:end,:))-double(cost(1:end-1,:));

Vp_ix = find(w(:)>=0); % positive vertices, to be connected to sink
Vm_ix = find(w(:)<0); % negative vertices, to be connected to source
Es = [Vm_ix, -w(Vm_ix), zeros(length(Vm_ix),1)]; % source edges
Et = [Vp_ix, zeros(length(Vp_ix),1), w(Vp_ix)]; % sink edges

Est = [Es;Et]; % all terminal edges

% FINDING GRAPH CUT USING MAGIC
Scut = GraphCutMex(R*N,Est,E);

% retreiving surfaces as the upper envelope of Scut
S = zeros(size(cost));
S(Scut) = 1;
s = zeros(N,1);
for n = 1:N;
    s(n) = find(S(:,n),1,'last');
end

disp(['finished at ',num2str(toc)])
end

function edges = connect_to_neighbors(indices,A,dr)

from = 1-min(dr,0):size(indices,1)-max(dr,0);
into = 1+max(dr,0):size(indices,1)+min(dr,0);

l = size(indices,1)-abs(dr); % length of from and into

% every nonzero element in A contributes with l edges
edges = zeros(numel(find(A>0))*l,2);

i = 1;
for n = 1:size(A,1)
    c = find(A(n,:)>0); % indices of neighboors
    v = numel(c); % valency
    edges(i:i+v*l-1,:) = [repmat(indices(from,n),[v,1]),...
        reshape(indices(into,c),[v*l,1])];
    i = i+v*l;
end
end

function edges = connect_verticaly(indices,dr)
% edges in vertical direction, does not need connectivity

from = 1-min(dr,0):size(indices,1)-max(dr,0);
into = 1+max(dr,0):size(indices,1)+min(dr,0);

E = size(indices,2)*(size(indices,1)-abs(dr));
edges = [reshape(indices(from,:),[E,1]),reshape(indices(into,:),[E,1])];

end
