function s = optimal_k_cut(cost_s,cost_r,A,delta_r,delta_lu)
% Finds an optimal cut (a surface) between lower and upper part of a volume.
% TODO explain default values in help text.
%
% COST_S is a size(R,N,K) surface cost function. It is a collection of
%       length R cost samples in N columns for K surfaces. A value
%       cost_s(r,n,k) is an inverse to likelihood of a sample (r,n)
%       containing surface k. When utilizing regional information surfaces
%       are labeled in increasing order, so that the first surface has the
%       smallest r values.
% COST_R is a size(R,N,K+1) regional cost function It is a collection of
%       length R cost samples in N columns for K+1 regions between the K
%       surfaces. Region 1 is up (and including) the first surface, region
%       K+1 is above K-th surface. A value cost_r(r,n,k) is an inverse to
%       likelihood of a sample (r,n) being in region k.
% A is a size (N,N) adjacency matrix for columns of cost.
% DELTA_R is a length K array a surface stiffness parameters.
% DELTA_LU is a size (K-1,2) array of lower and upper surface overlaps.
%       The constrain is: dl <= surf_k+1-surf_k <= du. For non intersecting
%       surfaces dl and du need to have a same sign. For ordered surfaces
%       we have dl>0 and du>dl.
% S is a size (N,K) array of r coordinates for the K segmented surfaces.
%
% Author: Vedrana Andersen, vand at dtu.dk
% Date: 30.5.2013.
%
% NOTE: needs Kolmogorovns implementation of the algorithm from
%		"An Experimental Comparison of Min-Cut/Max-Flow Algorithms for
%       Energy Minimization in Vision."
%		Yuri Boykov and Vladimir Kolmogorov. PAMI 2004

% ASSIGNING DEFAULTS NOT TESTED
tic, fprintf('Optimal k cut, ')
[cost_s,cost_r,delta_r,delta_lu] = assign_defaults(cost_s,cost_r,delta_r,delta_lu);
[R,N,K] = size(cost_s);
fprintf('N=%d, M=%d, K=%d ... ',N,R,K)

% assigning an unique index to each vertex in the graph fo first surface
indices = reshape(1:R*N,[R N]);
base = indices(1,:);
next_surface = R*N; % offset between surface k and k+1
layers = (0:(K-1))*next_surface; % offsets from surface 1 to all others

% EDGES WHICH ARE THE SAME FOR ALL SURFACES:
% intracolumn arcs pointing down
ea1 = connect_verticaly(indices,-1); % first surface
ea = repmat(ea1,[K 1]) + kron(layers(:),ones(size(ea1))); % all surfaces
Ea = [ea,ones(size(ea,1),1)*[inf,0]]; % assigning [inf 0] weight

% base edges, part of intercolumn arcs
eb1 = unique(sort(connect_to_neighbors(base,A,0),2),'rows'); % first surface
eb = repmat(eb1,[K 1]) + kron(layers(:),ones(size(eb1))); % all surfaces
% assigning [inf inf] weight, base edges are in both directions
Eb = [eb,ones(size(eb,1),1)*[inf,inf]];

% EDGES WHICH DEPEND ON SURFACE STIFFNESS:
% slanted edges, part of intercolumn arcs
ed = []; % depending on delta_r edk can have different lengths
for k = 1:K
    edk = connect_to_neighbors(indices,A,-delta_r(k)) + layers(k);
    ed = [ed; edk];
end
% assigning [inf 0] weight, slanted edges are in one direction
Ed = [ed,ones(size(ed,1),1)*[inf,0]];
E = [Ea; Eb; Ed]; % all intracolumn and intercolumn edges

% EDGES WHICH DEPEND ON DISTANCE BETWEEN SURFACES:
% intersurface arc
if K>1
    es = [];
    for k = 1:K-1
        esl = connect_verticaly(indices,delta_lu(k,1));
        esl = esl + ones(size(esl,1),1)*layers([k,k+1]);
        esu = connect_verticaly(indices,-delta_lu(k,2));
        esu = esu + ones(size(esu,1),1)*layers([k+1,k]);
        es = [es;esl;esu];        
    end
    Es = [es,ones(size(es,1),1)*[inf,0]]; % assigning [inf 0] weight
    
    % intersurface base edges
    esb = [layers(1:end-1)',layers(2:end)']+1; %first vertex in all surfaces
    Esb = [esb,ones(K-1,1)*[inf,inf]]; % assigning [inf inf]
    
    E = [E;Es;Esb]; % all intracolumn, intercolumn and intersurface edges, for all surfaces
end

% EDGES WHICH DEPEND ON SURFACE LIKELIHOOD:
% up to here we do not use cost -- consider efficient algorithm if solving
% multiple problems of the same size
% vertex cost done simulatniously for all surfaces
w_on = -1*ones(size(cost_s)); % to prevent empty solution, see second half of section 4.1
w_on(2:end,:,:) = double(cost_s(2:end,:,:))-double(cost_s(1:end-1,:,:));
% in region cost
w_in = cost_r(:,:,1:end-1) - cost_r(:,:,2:end);
w = w_on + w_in;

Vp_ix = find(w(:)>=0); % positive vertices, to be connected to sink
Vm_ix = find(w(:)<0); % negative vertices, to be connected to source
Es = [Vm_ix, -w(Vm_ix), zeros(length(Vm_ix),1)]; % source edges
Et = [Vp_ix, zeros(length(Vp_ix),1), w(Vp_ix)]; % sink edges

Est = [Es;Et]; % all terminal edges

% FINDING GRAPH CUT USING MAGIC
Scut = GraphCutMex(R*N*K,Est,E);

% retreiving surfaces as the upper envelope of Scut
S = zeros(R,N,K);
S(Scut) = 1;
s = zeros(N,K);
for n = 1:N;
    for k=1:K
        s(n,k) = find(S(:,n,k),1,'last');
    end
end

disp(['finished at ',num2str(toc)])
end

function edges = connect_to_neighbors(indices,A,dr)

from = 1-min(dr,0):size(indices,1)-max(dr,0);
into = 1+max(dr,0):size(indices,1)+min(dr,0);

l = size(indices,1)-abs(dr); % length of from and into

% every nonzero element in A contributes with l edges
edges = zeros(numel(find(A>0))*l,2);

i = 1;
for n = 1:size(A,1)
    c = find(A(n,:)>0); % indices of neighboors
    v = numel(c); % valency
    edges(i:i+v*l-1,:) = [repmat(indices(from,n),[v,1]),...
        reshape(indices(into,c),[v*l,1])];
    i = i+v*l;
end
end

function edges = connect_verticaly(indices,dr)
% edges in vertical direction, does not need connectivity

from = 1-min(dr,0):size(indices,1)-max(dr,0);
into = 1+max(dr,0):size(indices,1)+min(dr,0);

E = size(indices,2)*(size(indices,1)-abs(dr));
edges = [reshape(indices(from,:),[E,1]),reshape(indices(into,:),[E,1])];

end

function [cost_s,cost_r,delta_r,delta_lu] = assign_defaults(cost_s,cost_r,delta_r,delta_lu)

% either cost_s or cost_r need to be given
regional = false;
if all(cost_s(:)==0)
    regional = true;
end
if isempty(cost_s)
    regional = true;
    cost_s = zeros(size(cost_r)-[0 0 1]);    
elseif isempty(cost_r)
    cost_r = zeros(size(cost_s)+[0 0 1]);
end

[R,~,K] = size(cost_s);

% smoothness constraint may be given for all surfaces
if numel(delta_r)==1
    delta_r = delta_r*ones(K,1);
end

% overlap constraint defaults to no overlap in region case
% and no constraint in surface case
if isempty(delta_lu)
    if regional % no overlap
        delta_lu = [1 R];
    else % no constraint
        delta_lu = [-R R];
    end    
end

% overlap constraint may be given for all surface pairs
if all(size(delta_lu)==[1,2])
    delta_lu = ones(K-1,1)*delta_lu;
end

end
